"""Provide a client."""
import asyncio
import logging
from typing import Awaitable, Callable

import click

from ..client import Client
from ..exceptions import AIOVLCError
from ..model.command import Info, Status

LOGGER = logging.getLogger("aiovlc")

ClientFactory = Callable[[], Awaitable[Client]]


@click.command(options_metavar="<options>")
@click.option(
    "--password",
    prompt=True,
    hide_input=True,
    required=True,
    help="Password of the connection.",
)
@click.option(
    "-H",
    "--host",
    default="localhost",
    show_default=True,
    help="Host of the connection.",
)
@click.option(
    "-p",
    "--port",
    default=4212,
    show_default=True,
    type=int,
    help="Port of the connection.",
)
def client(password: str, host: str, port: int) -> None:
    """Start a client."""

    async def client_factory() -> Client:
        """Return a client."""
        vlc_client = Client(password, host=host, port=port)
        return vlc_client

    run_client(client_factory)


def run_client(client_factory: ClientFactory) -> None:
    """Run a client."""
    LOGGER.info("Starting client")
    try:
        asyncio.run(start_client(client_factory))
    except KeyboardInterrupt:
        pass
    finally:
        LOGGER.info("Exiting CLI")


async def start_client(client_factory: ClientFactory) -> None:
    """Start the client."""
    vlc_client = await client_factory()

    try:
        async with vlc_client:
            await vlc_client.login()
            await handle_client(vlc_client)
    except AIOVLCError as err:
        LOGGER.error("Error '%s'", err)


async def handle_client(vlc_client: Client) -> None:
    """Handle the client calls."""
    while True:
        command = Status()
        output = await command.send(vlc_client)
        LOGGER.debug("Received: %s", output)
        info_command = Info()
        info_output = await info_command.send(vlc_client)
        LOGGER.debug("Received: %s", info_output)
        await asyncio.sleep(10)



"""Provide a CLI for aiovlc."""
import logging

import click

from aiovlc import __version__

from .client_command import client

SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(
    options_metavar="", subcommand_metavar="<command>", context_settings=SETTINGS
)
@click.option("--debug", is_flag=True, help="Start aiovlc in debug mode.")
@click.version_option(__version__)
def cli(debug: bool) -> None:
    """Run aiovlc as an app for testing purposes."""
    if debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)


cli.add_command(client)



"""Set up the aiovlc package."""
from pathlib import Path

from setuptools import find_packages, setup

DESCRIPTION = "A package for talking to vlc over its telnet interface using asyncio."

PROJECT_DIR = Path(__file__).parent.resolve()
README_FILE = PROJECT_DIR / "README.md"
LONG_DESCRIPTION = README_FILE.read_text(encoding="utf-8")
VERSION = (PROJECT_DIR / "aiovlc" / "VERSION").read_text().strip()

REQUIRES = [
    "click",
]

setup(
    name="aiovlc",
    version=VERSION,
    url="https://github.com/MartinHjelmare/aiovlc",
    author="Martin Hjelmare",
    author_email="marhje52@gmail.com",
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    package_data={"aiovlc": ["py.typed"]},
    include_package_data=True,
    zip_safe=False,
    license="Apache-2.0",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    python_requires=">=3.8",
    install_requires=REQUIRES,
    entry_points={"console_scripts": ["aiovlc = aiovlc.cli:cli"]},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Home Automation",
    ],
)
